@@ -5,17 +5,15 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.manager;
 
 import java.io.PrintWriter;
@@ -37,7 +35,8 @@ import javax.management.ObjectInstance;
 import javax.management.ObjectName;
 import javax.servlet.http.HttpServletResponse;
 
-import org.apache.catalina.util.RequestUtil;
+import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.security.Escape;
 
 /**
  * This is a refactoring of the servlet to externalize
@@ -45,16 +44,14 @@ import org.apache.catalina.util.RequestUtil;
  * use XSLT, that is unnecessarily complex.
  *
  * @author Peter Lin
- *
  */
-
 public class StatusTransformer {
 
 
     // --------------------------------------------------------- Public Methods
 
 
-    public static void setContentType(HttpServletResponse response, 
+    public static void setContentType(HttpServletResponse response,
                                       int mode) {
         if (mode == 0){
             response.setContentType("text/html;charset="+Constants.CHARSET);
@@ -64,14 +61,24 @@ public class StatusTransformer {
     }
 
 
-    public static void writeHeader(PrintWriter writer, int mode) {
+    /**
+     * Write an HTML or XML header.
+     *
+     * @param writer the PrintWriter to use
+     * @param args Path prefix for URLs
+     * @param mode - 0 = HTML header, 1 = XML declaration
+     *
+     */
+    public static void writeHeader(PrintWriter writer, Object[] args, int mode) {
         if (mode == 0){
             // HTML Header Section
-            writer.print(Constants.HTML_HEADER_SECTION);
+            writer.print(MessageFormat.format(
+                Constants.HTML_HEADER_SECTION, args
+            ));
         } else if (mode == 1){
             writer.write(Constants.XML_DECLARATION);
-            writer.write
-                (Constants.XML_STYLE);
+            writer.print(MessageFormat.format
+                     (Constants.XML_STYLE, args));
             writer.write("<status>");
         }
     }
@@ -80,10 +87,10 @@ public class StatusTransformer {
     /**
      * Write the header body. XML output doesn't bother
      * to output this stuff, since it's just title.
-     * 
+     *
      * @param writer The output writer
      * @param args What to write
-     * @param mode 0 means write 
+     * @param mode 0 means write
      */
     public static void writeBody(PrintWriter writer, Object[] args, int mode) {
         if (mode == 0){
@@ -95,12 +102,12 @@ public class StatusTransformer {
 
     /**
      * Write the manager webapp information.
-     * 
+     *
      * @param writer The output writer
      * @param args What to write
      * @param mode 0 means write
      */
-    public static void writeManager(PrintWriter writer, Object[] args, 
+    public static void writeManager(PrintWriter writer, Object[] args,
                                     int mode) {
         if (mode == 0){
             writer.print(MessageFormat.format(Constants.MANAGER_SECTION, args));
@@ -108,7 +115,7 @@ public class StatusTransformer {
     }
 
 
-    public static void writePageHeading(PrintWriter writer, Object[] args, 
+    public static void writePageHeading(PrintWriter writer, Object[] args,
                                         int mode) {
         if (mode == 0){
             writer.print(MessageFormat.format
@@ -117,7 +124,7 @@ public class StatusTransformer {
     }
 
 
-    public static void writeServerInfo(PrintWriter writer, Object[] args, 
+    public static void writeServerInfo(PrintWriter writer, Object[] args,
                                        int mode){
         if (mode == 0){
             writer.print(MessageFormat.format(Constants.SERVER_ROW_SECTION, args));
@@ -125,9 +132,6 @@ public class StatusTransformer {
     }
 
 
-    /**
-     * 
-     */
     public static void writeFooter(PrintWriter writer, int mode) {
         if (mode == 0){
             // HTML Tail Section
@@ -139,15 +143,19 @@ public class StatusTransformer {
 
 
     /**
-     * Write the OS state. Mode 0 will generate HTML.
-     * Mode 1 will generate XML.
+     * Write the OS state.
+     *
+     * @param writer The output writer
+     * @param mode Mode <code>0</code> will generate HTML.
+     *             Mode <code>1</code> will generate XML.
+     * @param args I18n labels for the OS state values
      */
-    public static void writeOSState(PrintWriter writer, int mode) {
+    public static void writeOSState(PrintWriter writer, int mode, Object[] args) {
         long[] result = new long[16];
         boolean ok = false;
         try {
             String methodName = "info";
-            Class paramTypes[] = new Class[1];
+            Class<?> paramTypes[] = new Class[1];
             paramTypes[0] = result.getClass();
             Object paramValues[] = new Object[1];
             paramValues[0] = result;
@@ -156,46 +164,63 @@ public class StatusTransformer {
             method.invoke(null, paramValues);
             ok = true;
         } catch (Throwable t) {
-            // Ignore
+            t = ExceptionUtils.unwrapInvocationTargetException(t);
+            ExceptionUtils.handleThrowable(t);
         }
-        
+
         if (ok) {
             if (mode == 0){
                 writer.print("<h1>OS</h1>");
 
                 writer.print("<p>");
-                writer.print(" Physical memory: ");
-                writer.print(formatSize(new Long(result[0]), true));
-                writer.print(" Available memory: ");
-                writer.print(formatSize(new Long(result[1]), true));
-                writer.print(" Total page file: ");
-                writer.print(formatSize(new Long(result[2]), true));
-                writer.print(" Free page file: ");
-                writer.print(formatSize(new Long(result[3]), true));
-                writer.print(" Memory load: ");
-                writer.print(new Long(result[6]));
+                writer.print( args[0] );
+                writer.print(' ');
+                writer.print(formatSize(Long.valueOf(result[0]), true));
+                writer.print(' ');
+                writer.print(args[1]);
+                writer.print(' ');
+                writer.print(formatSize(Long.valueOf(result[1]), true));
+                writer.print(' ');
+                writer.print(args[2]);
+                writer.print(' ');
+                writer.print(formatSize(Long.valueOf(result[2]), true));
+                writer.print(' ');
+                writer.print(args[3]);
+                writer.print(' ');
+                writer.print(formatSize(Long.valueOf(result[3]), true));
+                writer.print(' ');
+                writer.print(args[4]);
+                writer.print(' ');
+                writer.print(Long.valueOf(result[6]));
                 writer.print("<br>");
-                writer.print(" Process kernel time: ");
-                writer.print(formatTime(new Long(result[11] / 1000), true));
-                writer.print(" Process user time: ");
-                writer.print(formatTime(new Long(result[12] / 1000), true));
+                writer.print(args[5]);
+                writer.print(' ');
+                writer.print(formatTime(Long.valueOf(result[11] / 1000), true));
+                writer.print(' ');
+                writer.print(args[6]);
+                writer.print(' ');
+                writer.print(formatTime(Long.valueOf(result[12] / 1000), true));
                 writer.print("</p>");
             } else if (mode == 1){
+                // NO-OP
             }
         }
-        
+
     }
-    
-    
+
+
     /**
-     * Write the VM state. Mode 0 will generate HTML.
-     * Mode 1 will generate XML.
+     * Write the VM state.
+     * @param writer The output writer
+     * @param mode Mode <code>0</code> will generate HTML.
+     *             Mode <code>1</code> will generate XML.
+     * @param args I18n labels for the VM state values
+     * @throws Exception Propagated JMX error
      */
-    public static void writeVMState(PrintWriter writer, int mode)
+    public static void writeVMState(PrintWriter writer, int mode, Object[] args)
         throws Exception {
 
-        SortedMap<String, MemoryPoolMXBean> memoryPoolMBeans =
-                new TreeMap<String, MemoryPoolMXBean>();
+        SortedMap<String, MemoryPoolMXBean> memoryPoolMBeans = new TreeMap<>();
         for (MemoryPoolMXBean mbean: ManagementFactory.getMemoryPoolMXBeans()) {
             String sortKey = mbean.getType() + ":" + mbean.getName();
             memoryPoolMBeans.put(sortKey, mbean);
@@ -205,18 +230,23 @@ public class StatusTransformer {
             writer.print("<h1>JVM</h1>");
 
             writer.print("<p>");
-            writer.print(" Free memory: ");
-            writer.print(formatSize
-                         (new Long(Runtime.getRuntime().freeMemory()), true));
-            writer.print(" Total memory: ");
-            writer.print(formatSize
-                         (new Long(Runtime.getRuntime().totalMemory()), true));
-            writer.print(" Max memory: ");
-            writer.print(formatSize
-                         (new Long(Runtime.getRuntime().maxMemory()), true));
+            writer.print( args[0] );
+            writer.print(' ');
+            writer.print(formatSize(
+                    Long.valueOf(Runtime.getRuntime().freeMemory()), true));
+            writer.print(' ');
+            writer.print(args[1]);
+            writer.print(' ');
+            writer.print(formatSize(
+                    Long.valueOf(Runtime.getRuntime().totalMemory()), true));
+            writer.print(' ');
+            writer.print(args[2]);
+            writer.print(' ');
+            writer.print(formatSize(
+                    Long.valueOf(Runtime.getRuntime().maxMemory()), true));
             writer.print("</p>");
 
-            writer.write("<table border=\"0\"><thead><tr><th>Memory Pool</th><th>Type</th><th>Initial</th><th>Total</th><th>Maximum</th><th>Used</th></tr></thead><tbody>");
+            writer.write("<table border=\"0\"><thead><tr><th>" + args[3] + "</th><th>" + args[4] + "</th><th>" + args[5] + "</th><th>" + args[6] + "</th><th>" + args[7] + "</th><th>" + args[8] + "</th></tr></thead><tbody>");
             for (MemoryPoolMXBean memoryPoolMBean : memoryPoolMBeans.values()) {
                 MemoryUsage usage = memoryPoolMBean.getUsage();
                 writer.write("<tr><td>");
@@ -249,7 +279,7 @@ public class StatusTransformer {
             for (MemoryPoolMXBean memoryPoolMBean : memoryPoolMBeans.values()) {
                 MemoryUsage usage = memoryPoolMBean.getUsage();
                 writer.write("<memorypool");
-                writer.write(" name='" + memoryPoolMBean.getName() + "'");
+                writer.write(" name='" + Escape.xml("", memoryPoolMBean.getName()) + "'");
                 writer.write(" type='" + memoryPoolMBean.getType() + "'");
                 writer.write(" usageInit='" + usage.getInit() + "'");
                 writer.write(" usageCommitted='" + usage.getCommitted() + "'");
@@ -265,14 +295,21 @@ public class StatusTransformer {
 
     /**
      * Write connector state.
+     * @param writer The output writer
+     * @param tpName MBean name of the thread pool
+     * @param name Connector name
+     * @param mBeanServer MBean server
+     * @param globalRequestProcessors MBean names for the global request processors
+     * @param requestProcessors MBean names for the request processors
+     * @param mode Mode <code>0</code> will generate HTML.
+     *             Mode <code>1</code> will generate XML.
+     * @param args I18n labels for the Connector state values
+     * @throws Exception Propagated JMX error
      */
-    public static void writeConnectorState(PrintWriter writer, 
-                                           ObjectName tpName, String name,
-                                           MBeanServer mBeanServer,
-                                           Vector globalRequestProcessors,
-                                           Vector requestProcessors,
-                                           int mode)
-        throws Exception {
+    public static void writeConnectorState(PrintWriter writer,
+            ObjectName tpName, String name, MBeanServer mBeanServer,
+            Vector<ObjectName> globalRequestProcessors,
+            Vector<ObjectName> requestProcessors, int mode, Object[] args) throws Exception {
 
         if (mode == 0) {
             writer.print("<h1>");
@@ -280,28 +317,31 @@ public class StatusTransformer {
             writer.print("</h1>");
 
             writer.print("<p>");
-            writer.print(" Max threads: ");
+            writer.print( args[0] );
+            writer.print(' ');
             writer.print(mBeanServer.getAttribute(tpName, "maxThreads"));
-            writer.print(" Current thread count: ");
+            writer.print(' ');
+            writer.print(args[1]);
+            writer.print(' ');
             writer.print(mBeanServer.getAttribute(tpName, "currentThreadCount"));
-            writer.print(" Current thread busy: ");
+            writer.print(' ');
+            writer.print(args[2]);
+            writer.print(' ');
             writer.print(mBeanServer.getAttribute(tpName, "currentThreadsBusy"));
-            try {
-                Object value = mBeanServer.getAttribute(tpName, "keepAliveCount");
-                writer.print(" Keep alive sockets count: ");
-                writer.print(value);
-            } catch (Exception e) {
-                // Ignore
-            }
-            
+            writer.print(' ');
+            writer.print(args[3]);
+            writer.print(' ');
+            writer.print(mBeanServer.getAttribute(tpName, "keepAliveCount"));
+
             writer.print("<br>");
 
             ObjectName grpName = null;
 
-            Enumeration enumeration = globalRequestProcessors.elements();
+            Enumeration<ObjectName> enumeration = globalRequestProcessors.elements();
+            // Find the HTTP/1.1 RequestGroupInfo - BZ 65404
             while (enumeration.hasMoreElements()) {
-                ObjectName objectName = (ObjectName) enumeration.nextElement();
-                if (name.equals(objectName.getKeyProperty("name"))) {
+                ObjectName objectName = enumeration.nextElement();
+                if (name.equals(objectName.getKeyProperty("name")) && objectName.getKeyProperty("Upgrade") == null) {
                     grpName = objectName;
                 }
             }
@@ -310,29 +350,40 @@ public class StatusTransformer {
                 return;
             }
 
-            writer.print(" Max processing time: ");
+            writer.print( args[4] );
+            writer.print(' ');
             writer.print(formatTime(mBeanServer.getAttribute
                                     (grpName, "maxTime"), false));
-            writer.print(" Processing time: ");
+            writer.print(' ');
+            writer.print(args[5]);
+            writer.print(' ');
             writer.print(formatTime(mBeanServer.getAttribute
                                     (grpName, "processingTime"), true));
-            writer.print(" Request count: ");
+            writer.print(' ');
+            writer.print(args[6]);
+            writer.print(' ');
             writer.print(mBeanServer.getAttribute(grpName, "requestCount"));
-            writer.print(" Error count: ");
+            writer.print(' ');
+            writer.print(args[7]);
+            writer.print(' ');
             writer.print(mBeanServer.getAttribute(grpName, "errorCount"));
-            writer.print(" Bytes received: ");
+            writer.print(' ');
+            writer.print(args[8]);
+            writer.print(' ');
             writer.print(formatSize(mBeanServer.getAttribute
                                     (grpName, "bytesReceived"), true));
-            writer.print(" Bytes sent: ");
+            writer.print(' ');
+            writer.print(args[9]);
+            writer.print(' ');
             writer.print(formatSize(mBeanServer.getAttribute
                                     (grpName, "bytesSent"), true));
             writer.print("</p>");
 
-            writer.print("<table border=\"0\"><tr><th>Stage</th><th>Time</th><th>B Sent</th><th>B Recv</th><th>Client</th><th>VHost</th><th>Request</th></tr>");
+            writer.print("<table border=\"0\"><tr><th>"+ args[10] + "</th><th>" + args[11] + "</th><th>" + args[12] +"</th><th>" + args[13] +"</th><th>" + args[14] + "</th><th>" + args[15] + "</th><th>" + args[16] + "</th><th>" + args[17] + "</th></tr>");
 
             enumeration = requestProcessors.elements();
             while (enumeration.hasMoreElements()) {
-                ObjectName objectName = (ObjectName) enumeration.nextElement();
+                ObjectName objectName = enumeration.nextElement();
                 if (name.equals(objectName.getKeyProperty("worker"))) {
                     writer.print("<tr>");
                     writeProcessorState(writer, objectName, mBeanServer, mode);
@@ -343,7 +394,7 @@ public class StatusTransformer {
             writer.print("</table>");
 
             writer.print("<p>");
-            writer.print("P: Parse and prepare request S: Service F: Finishing R: Ready K: Keepalive");
+            writer.print( args[18] );
             writer.print("</p>");
         } else if (mode == 1){
             writer.write("<connector name='" + name + "'>");
@@ -356,10 +407,11 @@ public class StatusTransformer {
 
             ObjectName grpName = null;
 
-            Enumeration enumeration = globalRequestProcessors.elements();
+            Enumeration<ObjectName> enumeration = globalRequestProcessors.elements();
+            // Find the HTTP/1.1 RequestGroupInfo - BZ 65404
             while (enumeration.hasMoreElements()) {
-                ObjectName objectName = (ObjectName) enumeration.nextElement();
-                if (name.equals(objectName.getKeyProperty("name"))) {
+                ObjectName objectName = enumeration.nextElement();
+                if (name.equals(objectName.getKeyProperty("name")) && objectName.getKeyProperty("Upgrade") == null) {
                     grpName = objectName;
                 }
             }
@@ -378,7 +430,7 @@ public class StatusTransformer {
                 writer.write("<workers>");
                 enumeration = requestProcessors.elements();
                 while (enumeration.hasMoreElements()) {
-                    ObjectName objectName = (ObjectName) enumeration.nextElement();
+                    ObjectName objectName = enumeration.nextElement();
                     if (name.equals(objectName.getKeyProperty("worker"))) {
                         writeProcessorState(writer, objectName, mBeanServer, mode);
                     }
@@ -394,14 +446,20 @@ public class StatusTransformer {
 
     /**
      * Write processor state.
+     * @param writer The output writer
+     * @param pName MBean name of the processor
+     * @param  mBeanServer MBean server
+     * @param mode Mode <code>0</code> will generate HTML.
+     *   Mode <code>1</code> will generate XML.
+     * @throws Exception Propagated JMX error
      */
-    protected static void writeProcessorState(PrintWriter writer, 
+    protected static void writeProcessorState(PrintWriter writer,
                                               ObjectName pName,
-                                              MBeanServer mBeanServer, 
+                                              MBeanServer mBeanServer,
                                               int mode)
         throws Exception {
 
-        Integer stageValue = 
+        Integer stageValue =
             (Integer) mBeanServer.getAttribute(pName, "stage");
         int stage = stageValue.intValue();
         boolean fullStatus = true;
@@ -468,35 +526,39 @@ public class StatusTransformer {
                 writer.write("<td>");
                 if (showRequest) {
                     writer.print(formatSize(mBeanServer.getAttribute
-                                            (pName, "requestBytesReceived"), 
+                                            (pName, "requestBytesReceived"),
                                             false));
                 } else {
                     writer.write("?");
                 }
                 writer.write("</td>");
                 writer.write("<td>");
-                writer.print(filter(mBeanServer.getAttribute
+                writer.print(Escape.htmlElementContext(mBeanServer.getAttribute
+                                    (pName, "remoteAddrForwarded")));
+                writer.write("</td>");
+                writer.write("<td>");
+                writer.print(Escape.htmlElementContext(mBeanServer.getAttribute
                                     (pName, "remoteAddr")));
                 writer.write("</td>");
                 writer.write("<td nowrap>");
-                writer.write(filter(mBeanServer.getAttribute
+                writer.write(Escape.htmlElementContext(mBeanServer.getAttribute
                                     (pName, "virtualHost")));
                 writer.write("</td>");
-                writer.write("<td nowrap>");
+                writer.write("<td nowrap class=\"row-left\">");
                 if (showRequest) {
-                    writer.write(filter(mBeanServer.getAttribute
+                    writer.write(Escape.htmlElementContext(mBeanServer.getAttribute
                                         (pName, "method")));
-                    writer.write(" ");
-                    writer.write(filter(mBeanServer.getAttribute
+                    writer.write(' ');
+                    writer.write(Escape.htmlElementContext(mBeanServer.getAttribute
                                         (pName, "currentUri")));
                     String queryString = (String) mBeanServer.getAttribute
                         (pName, "currentQueryString");
                     if ((queryString != null) && (!queryString.equals(""))) {
                         writer.write("?");
-                        writer.print(RequestUtil.filter(queryString));
+                        writer.print(Escape.htmlElementContent(queryString));
                     }
-                    writer.write(" ");
-                    writer.write(filter(mBeanServer.getAttribute
+                    writer.write(' ');
+                    writer.write(Escape.htmlElementContext(mBeanServer.getAttribute
                                         (pName, "protocol")));
                 } else {
                     writer.write("?");
@@ -510,7 +572,7 @@ public class StatusTransformer {
             writer.write(" stage=\"" + stageStr + "\"");
 
             if (fullStatus) {
-                writer.write(" requestProcessingTime=\"" 
+                writer.write(" requestProcessingTime=\""
                              + mBeanServer.getAttribute
                              (pName, "requestProcessingTime") + "\"");
                 writer.write(" requestBytesSent=\"");
@@ -529,31 +591,31 @@ public class StatusTransformer {
                     writer.write("0");
                 }
                 writer.write("\"");
-                writer.write(" remoteAddr=\"" 
-                             + filter(mBeanServer.getAttribute
+                writer.write(" remoteAddr=\""
+                             + Escape.htmlElementContext(mBeanServer.getAttribute
                                       (pName, "remoteAddr")) + "\"");
-                writer.write(" virtualHost=\"" 
-                             + filter(mBeanServer.getAttribute
+                writer.write(" virtualHost=\""
+                             + Escape.htmlElementContext(mBeanServer.getAttribute
                                       (pName, "virtualHost")) + "\"");
 
                 if (showRequest) {
-                    writer.write(" method=\"" 
-                                 + filter(mBeanServer.getAttribute
+                    writer.write(" method=\""
+                                 + Escape.htmlElementContext(mBeanServer.getAttribute
                                           (pName, "method")) + "\"");
-                    writer.write(" currentUri=\"" 
-                                 + filter(mBeanServer.getAttribute
+                    writer.write(" currentUri=\""
+                                 + Escape.htmlElementContext(mBeanServer.getAttribute
                                           (pName, "currentUri")) + "\"");
 
                     String queryString = (String) mBeanServer.getAttribute
                         (pName, "currentQueryString");
                     if ((queryString != null) && (!queryString.equals(""))) {
-                        writer.write(" currentQueryString=\"" 
-                                     + RequestUtil.filter(queryString) + "\"");
+                        writer.write(" currentQueryString=\""
+                                     + Escape.htmlElementContent(queryString) + "\"");
                     } else {
                         writer.write(" currentQueryString=\"&#63;\"");
                     }
-                    writer.write(" protocol=\"" 
-                                 + filter(mBeanServer.getAttribute
+                    writer.write(" protocol=\""
+                                 + Escape.htmlElementContext(mBeanServer.getAttribute
                                           (pName, "protocol")) + "\"");
                 } else {
                     writer.write(" method=\"&#63;\"");
@@ -580,6 +642,11 @@ public class StatusTransformer {
 
     /**
      * Write applications state.
+     * @param writer The output writer
+     * @param mBeanServer MBean server
+     * @param mode Mode <code>0</code> will generate HTML.
+     *   Mode <code>1</code> will generate XML.
+     * @throws Exception Propagated JMX error
      */
     public static void writeDetailedState(PrintWriter writer,
                                           MBeanServer mBeanServer, int mode)
@@ -587,7 +654,7 @@ public class StatusTransformer {
 
         if (mode == 0){
             ObjectName queryHosts = new ObjectName("*:j2eeType=WebModule,*");
-            Set hostsON = mBeanServer.queryNames(queryHosts, null);
+            Set<ObjectName> hostsON = mBeanServer.queryNames(queryHosts, null);
 
             // Navigation menu
             writer.print("<h1>");
@@ -596,21 +663,21 @@ public class StatusTransformer {
 
             writer.print("<p>");
             int count = 0;
-            Iterator iterator = hostsON.iterator();
+            Iterator<ObjectName> iterator = hostsON.iterator();
             while (iterator.hasNext()) {
-                ObjectName contextON = (ObjectName) iterator.next();
+                ObjectName contextON = iterator.next();
                 String webModuleName = contextON.getKeyProperty("name");
                 if (webModuleName.startsWith("//")) {
                     webModuleName = webModuleName.substring(2);
                 }
-                int slash = webModuleName.indexOf("/");
+                int slash = webModuleName.indexOf('/');
                 if (slash == -1) {
                     count++;
                     continue;
                 }
 
                 writer.print("<a href=\"#" + (count++) + ".0\">");
-                writer.print(filter(webModuleName));
+                writer.print(Escape.htmlElementContext(webModuleName));
                 writer.print("</a>");
                 if (iterator.hasNext()) {
                     writer.print("<br>");
@@ -623,8 +690,8 @@ public class StatusTransformer {
             count = 0;
             iterator = hostsON.iterator();
             while (iterator.hasNext()) {
-                ObjectName contextON = (ObjectName) iterator.next();
-                writer.print("<a class=\"A.name\" name=\"" 
+                ObjectName contextON = iterator.next();
+                writer.print("<a class=\"A.name\" name=\""
                              + (count++) + ".0\">");
                 writeContext(writer, contextON, mBeanServer, mode);
             }
@@ -638,8 +705,14 @@ public class StatusTransformer {
 
     /**
      * Write context state.
+     * @param writer The output writer
+     * @param objectName The context MBean name
+     * @param mBeanServer MBean server
+     * @param mode Mode <code>0</code> will generate HTML.
+     *   Mode <code>1</code> will generate XML.
+     * @throws Exception Propagated JMX error
      */
-    protected static void writeContext(PrintWriter writer, 
+    protected static void writeContext(PrintWriter writer,
                                        ObjectName objectName,
                                        MBeanServer mBeanServer, int mode)
         throws Exception {
@@ -650,13 +723,13 @@ public class StatusTransformer {
             if (name == null) {
                 return;
             }
-            
+
             String hostName = null;
             String contextName = null;
             if (name.startsWith("//")) {
                 name = name.substring(2);
             }
-            int slash = name.indexOf("/");
+            int slash = name.indexOf('/');
             if (slash != -1) {
                 hostName = name.substring(0, slash);
                 contextName = name.substring(slash);
@@ -665,19 +738,20 @@ public class StatusTransformer {
             }
 
             ObjectName queryManager = new ObjectName
-                (objectName.getDomain() + ":type=Manager,path=" + contextName 
+                (objectName.getDomain() + ":type=Manager,context=" + contextName
                  + ",host=" + hostName + ",*");
-            Set managersON = mBeanServer.queryNames(queryManager, null);
+            Set<ObjectName> managersON =
+                mBeanServer.queryNames(queryManager, null);
             ObjectName managerON = null;
-            Iterator iterator2 = managersON.iterator();
-            while (iterator2.hasNext()) {
-                managerON = (ObjectName) iterator2.next();
+            for (ObjectName aManagersON : managersON) {
+                managerON = aManagersON;
             }
 
             ObjectName queryJspMonitor = new ObjectName
                 (objectName.getDomain() + ":type=JspMonitor,WebModule=" +
                  webModuleName + ",*");
-            Set jspMonitorONs = mBeanServer.queryNames(queryJspMonitor, null);
+            Set<ObjectName> jspMonitorONs =
+                mBeanServer.queryNames(queryJspMonitor, null);
 
             // Special case for the root context
             if (contextName.equals("/")) {
@@ -685,7 +759,7 @@ public class StatusTransformer {
             }
 
             writer.print("<h1>");
-            writer.print(filter(name));
+            writer.print(Escape.htmlElementContext(name));
             writer.print("</h1>");
             writer.print("</a>");
 
@@ -708,13 +782,12 @@ public class StatusTransformer {
             }
             writer.print("</p>");
 
-            String onStr = objectName.getDomain() 
+            String onStr = objectName.getDomain()
                 + ":j2eeType=Servlet,WebModule=" + webModuleName + ",*";
             ObjectName servletObjectName = new ObjectName(onStr);
-            Set set = mBeanServer.queryMBeans(servletObjectName, null);
-            Iterator iterator = set.iterator();
-            while (iterator.hasNext()) {
-                ObjectInstance oi = (ObjectInstance) iterator.next();
+            Set<ObjectInstance> set =
+                mBeanServer.queryMBeans(servletObjectName, null);
+            for (ObjectInstance oi : set) {
                 writeWrapper(writer, oi.getObjectName(), mBeanServer, mode);
             }
 
@@ -727,6 +800,12 @@ public class StatusTransformer {
 
     /**
      * Write detailed information about a manager.
+     * @param writer The output writer
+     * @param objectName The manager MBean name
+     * @param mBeanServer MBean server
+     * @param mode Mode <code>0</code> will generate HTML.
+     *   Mode <code>1</code> will generate XML.
+     * @throws Exception Propagated JMX error
      */
     public static void writeManager(PrintWriter writer, ObjectName objectName,
                                     MBeanServer mBeanServer, int mode)
@@ -768,9 +847,15 @@ public class StatusTransformer {
 
     /**
      * Write JSP monitoring information.
+     * @param writer The output writer
+     * @param jspMonitorONs The JSP MBean names
+     * @param mBeanServer MBean server
+     * @param mode Mode <code>0</code> will generate HTML.
+     *   Mode <code>1</code> will generate XML.
+     * @throws Exception Propagated JMX error
      */
     public static void writeJspMonitor(PrintWriter writer,
-                                       Set jspMonitorONs,
+                                       Set<ObjectName> jspMonitorONs,
                                        MBeanServer mBeanServer,
                                        int mode)
             throws Exception {
@@ -778,9 +863,7 @@ public class StatusTransformer {
         int jspCount = 0;
         int jspReloadCount = 0;
 
-        Iterator iter = jspMonitorONs.iterator();
-        while (iter.hasNext()) {
-            ObjectName jspMonitorON = (ObjectName) iter.next();
+        for (ObjectName jspMonitorON : jspMonitorONs) {
             Object obj = mBeanServer.getAttribute(jspMonitorON, "jspCount");
             jspCount += ((Integer) obj).intValue();
             obj = mBeanServer.getAttribute(jspMonitorON, "jspReloadCount");
@@ -801,6 +884,12 @@ public class StatusTransformer {
 
     /**
      * Write detailed information about a wrapper.
+     * @param writer The output writer
+     * @param objectName The wrapper MBean names
+     * @param mBeanServer MBean server
+     * @param mode Mode <code>0</code> will generate HTML.
+     *   Mode <code>1</code> will generate XML.
+     * @throws Exception Propagated JMX error
      */
     public static void writeWrapper(PrintWriter writer, ObjectName objectName,
                                     MBeanServer mBeanServer, int mode)
@@ -808,16 +897,16 @@ public class StatusTransformer {
 
         if (mode == 0) {
             String servletName = objectName.getKeyProperty("name");
-            
-            String[] mappings = (String[]) 
+
+            String[] mappings = (String[])
                 mBeanServer.invoke(objectName, "findMappings", null, null);
-            
+
             writer.print("<h2>");
-            writer.print(filter(servletName));
+            writer.print(Escape.htmlElementContext(servletName));
             if ((mappings != null) && (mappings.length > 0)) {
                 writer.print(" [ ");
                 for (int i = 0; i < mappings.length; i++) {
-                    writer.print(filter(mappings[i]));
+                    writer.print(Escape.htmlElementContext(mappings[i]));
                     if (i < mappings.length - 1) {
                         writer.print(" , ");
                     }
@@ -825,7 +914,7 @@ public class StatusTransformer {
                 writer.print(" ] ");
             }
             writer.print("</h2>");
-            
+
             writer.print("<p>");
             writer.print(" Processing time: ");
             writer.print(formatTime(mBeanServer.getAttribute
@@ -857,16 +946,21 @@ public class StatusTransformer {
      * codes in the request URL that is often reported in error messages.
      *
      * @param obj The message string to be filtered
+     * @return filtered HTML content
+     *
+     * @deprecated This method will be removed in Tomcat 9
      */
+    @Deprecated
     public static String filter(Object obj) {
 
-        if (obj == null)
+        if (obj == null) {
             return ("?");
+        }
         String message = obj.toString();
 
         char content[] = new char[message.length()];
         message.getChars(0, message.length(), content, 0);
-        StringBuffer result = new StringBuffer(content.length + 50);
+        StringBuilder result = new StringBuilder(content.length + 50);
         for (int i = 0; i < content.length; i++) {
             switch (content[i]) {
             case '<':
@@ -885,15 +979,50 @@ public class StatusTransformer {
                 result.append(content[i]);
             }
         }
-        return (result.toString());
+        return result.toString();
 
     }
 
 
     /**
-     * Display the given size in bytes, either as KB or MB.
+     * Escape the 5 entities defined by XML.
+     * @param s The message string to be filtered
+     * @return filtered XML content
      *
-     * @param mb true to display megabytes, false for kilobytes
+     * @deprecated This method will be removed in Tomcat 9
+     */
+    @Deprecated
+    public static String filterXml(String s) {
+        if (s == null) {
+            return "";
+        }
+        StringBuilder sb = new StringBuilder();
+        for (int i = 0; i < s.length(); i++) {
+            char c = s.charAt(i);
+            if (c == '<') {
+                sb.append("&lt;");
+            } else if (c == '>') {
+                sb.append("&gt;");
+            } else if (c == '\'') {
+                sb.append("&apos;");
+            } else if (c == '&') {
+                sb.append("&amp;");
+            } else if (c == '"') {
+                sb.append("&quot;");
+            } else {
+                sb.append(c);
+            }
+        }
+        return sb.toString();
+    }
+
+
+    /**
+     * Display the given size in bytes, either as KiB or MiB.
+     *
+     * @param obj The object to format
+     * @param mb true to display MiB, false for KiB
+     * @return formatted size
      */
     public static String formatSize(Object obj, boolean mb) {
 
@@ -912,16 +1041,16 @@ public class StatusTransformer {
                 bytes = -bytes;
             }
             long mbytes = bytes / (1024 * 1024);
-            long rest = 
+            long rest =
                 ((bytes - (mbytes * (1024 * 1024))) * 100) / (1024 * 1024);
             buff.append(mbytes).append('.');
             if (rest < 10) {
                 buff.append('0');
             }
-            buff.append(rest).append(" MB");
+            buff.append(rest).append(" MiB");
             return buff.toString();
         } else {
-            return ((bytes / 1024) + " KB");
+            return ((bytes / 1024) + " KiB");
         }
 
     }
@@ -930,7 +1059,9 @@ public class StatusTransformer {
     /**
      * Display the given time in ms, either as ms or s.
      *
+     * @param obj The object to format
      * @param seconds true to display seconds, false for milliseconds
+     * @return formatted time
      */
     public static String formatTime(Object obj, boolean seconds) {
 
@@ -954,8 +1085,7 @@ public class StatusTransformer {
      * Formats the given time (given in seconds) as a string.
      *
      * @param obj Time object to be formatted as string
-     *
-     * @return String formatted time
+     * @return formatted time
      */
     public static String formatSeconds(Object obj) {
 
