@@ -14,13 +14,14 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.catalina.tribes.membership;
 
 import java.io.IOException;
 import java.net.DatagramPacket;
 import java.util.Properties;
 
+import javax.management.ObjectName;
+
 import org.apache.catalina.tribes.Channel;
 import org.apache.catalina.tribes.ChannelException;
 import org.apache.catalina.tribes.ChannelMessage;
@@ -30,9 +31,12 @@ import org.apache.catalina.tribes.MembershipService;
 import org.apache.catalina.tribes.MessageListener;
 import org.apache.catalina.tribes.io.ChannelData;
 import org.apache.catalina.tribes.io.XByteBuffer;
+import org.apache.catalina.tribes.jmx.JmxRegistry;
 import org.apache.catalina.tribes.util.Arrays;
 import org.apache.catalina.tribes.util.StringManager;
 import org.apache.catalina.tribes.util.UUIDGenerator;
+import org.apache.juli.logging.Log;
+import org.apache.juli.logging.LogFactory;
 
 /**
  * A <b>membership</b> implementation using simple multicast.
@@ -40,10 +44,10 @@ import org.apache.catalina.tribes.util.UUIDGenerator;
  * This class is responsible for maintaining a list of active cluster nodes in the cluster.
  * If a node fails to send out a heartbeat, the node will be dismissed.
  */
-public class McastService implements MembershipService,MembershipListener,MessageListener {
+public class McastService
+        implements MembershipService,MembershipListener,MessageListener, McastServiceMBean {
 
-    private static final org.apache.juli.logging.Log log =
-        org.apache.juli.logging.LogFactory.getLog( McastService.class );
+    private static final Log log = LogFactory.getLog(McastService.class);
 
     /**
      * The string manager for this package.
@@ -79,6 +83,11 @@ public class McastService implements MembershipService,MembershipListener,Messag
 
     private Channel channel;
 
+    /**
+     * the ObjectName of this McastService.
+     */
+    private ObjectName oname = null;
+
     /**
      * Create a membership service.
      */
@@ -88,7 +97,7 @@ public class McastService implements MembershipService,MembershipListener,Messag
     }
 
     /**
-     *
+     * Sets the properties for the membership service.
      * @param properties
      * <br>All are required<br>
      * 1. mcastPort - the port to listen to<BR>
@@ -112,33 +121,27 @@ public class McastService implements MembershipService,MembershipListener,Messag
         this.properties = properties;
     }
 
-    /**
-     * Return the properties, see setProperties
-     */
     @Override
     public Properties getProperties() {
         return properties;
     }
 
     /**
-     * Return the local member name
+     * @return the local member name
      */
+    @Override
     public String getLocalMemberName() {
         return localMember.toString() ;
     }
 
-    /**
-     * Return the local member
-     */
     @Override
     public Member getLocalMember(boolean alive) {
-        if ( alive && localMember != null && impl != null) localMember.setMemberAliveTime(System.currentTimeMillis()-impl.getServiceStartTime());
+        if ( alive && localMember != null && impl != null) {
+            localMember.setMemberAliveTime(System.currentTimeMillis()-impl.getServiceStartTime());
+        }
         return localMember;
     }
 
-    /**
-     * Sets the local member properties for broadcasting
-     */
     @Override
     public void setLocalMemberProperties(String listenHost, int listenPort, int securePort, int udpPort) {
         properties.setProperty("tcpListenHost",listenHost);
@@ -168,6 +171,7 @@ public class McastService implements MembershipService,MembershipListener,Messag
         properties.setProperty("mcastAddress", addr);
     }
 
+    @Override
     public String getAddress() {
         return properties.getProperty("mcastAddress");
     }
@@ -180,6 +184,7 @@ public class McastService implements MembershipService,MembershipListener,Messag
         properties.setProperty("mcastBindAddress", bindaddr);
     }
 
+    @Override
     public String getBind() {
         return properties.getProperty("mcastBindAddress");
     }
@@ -192,6 +197,7 @@ public class McastService implements MembershipService,MembershipListener,Messag
         properties.setProperty("recoveryCounter", String.valueOf(recoveryCounter));
     }
 
+    @Override
     public int getRecoveryCounter(){
         String p = properties.getProperty("recoveryCounter");
         if(p != null){
@@ -204,6 +210,7 @@ public class McastService implements MembershipService,MembershipListener,Messag
         properties.setProperty("recoveryEnabled", String.valueOf(recoveryEnabled));
     }
 
+    @Override
     public boolean getRecoveryEnabled() {
         String p = properties.getProperty("recoveryEnabled");
         if(p != null){
@@ -216,6 +223,7 @@ public class McastService implements MembershipService,MembershipListener,Messag
         properties.setProperty("recoverySleepTime", String.valueOf(recoverySleepTime));
     }
 
+    @Override
     public long getRecoverySleepTime(){
         String p = properties.getProperty("recoverySleepTime");
         if(p != null){
@@ -228,6 +236,7 @@ public class McastService implements MembershipService,MembershipListener,Messag
         properties.setProperty("localLoopbackDisabled",String.valueOf(localLoopbackDisabled));
     }
 
+    @Override
     public boolean getLocalLoopbackDisabled() {
         String p = properties.getProperty("localLoopbackDisabled");
         if(p != null){
@@ -236,6 +245,7 @@ public class McastService implements MembershipService,MembershipListener,Messag
         return false;
     }
 
+    @Override
     public int getPort() {
         String p = properties.getProperty("mcastPort");
         return Integer.parseInt(p);
@@ -245,6 +255,7 @@ public class McastService implements MembershipService,MembershipListener,Messag
         properties.setProperty("mcastFrequency", String.valueOf(time));
     }
 
+    @Override
     public long getFrequency() {
         String p = properties.getProperty("mcastFrequency");
         return Long.parseLong(p);
@@ -257,6 +268,7 @@ public class McastService implements MembershipService,MembershipListener,Messag
         properties.setProperty("memberDropTime", String.valueOf(time));
     }
 
+    @Override
     public long getDropTime() {
         String p = properties.getProperty("memberDropTime");
         return Long.parseLong(p);
@@ -268,7 +280,9 @@ public class McastService implements MembershipService,MembershipListener,Messag
      * @param name The property to check for
      */
     protected void hasProperty(Properties properties, String name){
-        if ( properties.getProperty(name)==null) throw new IllegalArgumentException(sm.getString("mcastService.missing.property", name));
+        if ( properties.getProperty(name)==null) {
+            throw new IllegalArgumentException(sm.getString("mcastService.missing.property", name));
+        }
     }
 
     /**
@@ -313,8 +327,12 @@ public class McastService implements MembershipService,MembershipListener,Messag
         }
         localMember.setSecurePort(securePort);
         localMember.setUdpPort(udpPort);
-        if ( this.payload != null ) localMember.setPayload(payload);
-        if ( this.domain != null ) localMember.setDomain(domain);
+        if ( this.payload != null ) {
+            localMember.setPayload(payload);
+        }
+        if ( this.domain != null ) {
+            localMember.setDomain(domain);
+        }
         localMember.setServiceStartTime(System.currentTimeMillis());
         java.net.InetAddress bind = null;
         if ( properties.getProperty("mcastBindAddress")!= null ) {
@@ -359,7 +377,11 @@ public class McastService implements MembershipService,MembershipListener,Messag
         impl.setChannel(channel);
 
         impl.start(level);
-
+        // register jmx
+        JmxRegistry jmxRegistry = JmxRegistry.getRegistry(channel);
+        if (jmxRegistry != null) {
+            this.oname = jmxRegistry.registerJmx(",component=Membership", this);
+        }
 
     }
 
@@ -371,6 +393,10 @@ public class McastService implements MembershipService,MembershipListener,Messag
     public void stop(int svc) {
         try  {
             if ( impl != null && impl.stop(svc) ) {
+                if (oname != null) {
+                    JmxRegistry.getRegistry(channel).unregisterJmx(oname);
+                    oname = null;
+                }
                 impl.setChannel(null);
                 impl = null;
                 channel = null;
@@ -394,8 +420,9 @@ public class McastService implements MembershipService,MembershipListener,Messag
             for (int i = 0; i < currentMembers.length; i++) {
                 membernames[i] = currentMembers[i].toString() ;
             }
-        } else
+        } else {
             membernames = new String[0] ;
+        }
         return membernames ;
     }
 
@@ -406,8 +433,9 @@ public class McastService implements MembershipService,MembershipListener,Messag
     public Member findMemberByName(String name) {
         Member[] currentMembers = getMembers();
         for (int i = 0; i < currentMembers.length; i++) {
-            if (name.equals(currentMembers[i].toString()))
+            if (name.equals(currentMembers[i].toString())) {
                 return currentMembers[i];
+            }
         }
         return null;
     }
@@ -417,13 +445,17 @@ public class McastService implements MembershipService,MembershipListener,Messag
      */
     @Override
     public boolean hasMembers() {
-       if ( impl == null || impl.membership == null ) return false;
+       if ( impl == null || impl.membership == null ) {
+        return false;
+    }
        return impl.membership.hasMembers();
     }
 
     @Override
     public Member getMember(Member mbr) {
-        if ( impl == null || impl.membership == null ) return null;
+        if ( impl == null || impl.membership == null ) {
+            return null;
+        }
         return impl.membership.getMember(mbr);
     }
 
@@ -433,7 +465,9 @@ public class McastService implements MembershipService,MembershipListener,Messag
     protected static final Member[]EMPTY_MEMBERS = new Member[0];
     @Override
     public Member[] getMembers() {
-        if ( impl == null || impl.membership == null ) return EMPTY_MEMBERS;
+        if ( impl == null || impl.membership == null ) {
+            return EMPTY_MEMBERS;
+        }
         return impl.membership.getMembers();
     }
     /**
@@ -483,7 +517,9 @@ public class McastService implements MembershipService,MembershipListener,Messag
 
     @Override
     public void messageReceived(ChannelMessage msg) {
-        if (msglistener!=null && msglistener.accept(msg)) msglistener.messageReceived(msg);
+        if (msglistener!=null && msglistener.accept(msg)) {
+            msglistener.messageReceived(msg);
+        }
     }
 
     @Override
@@ -492,8 +528,9 @@ public class McastService implements MembershipService,MembershipListener,Messag
     }
     @Override
     public void broadcast(ChannelMessage message) throws ChannelException {
-        if (impl==null || (impl.startLevel & Channel.MBR_TX_SEQ)!=Channel.MBR_TX_SEQ )
+        if (impl==null || (impl.startLevel & Channel.MBR_TX_SEQ)!=Channel.MBR_TX_SEQ ) {
             throw new ChannelException(sm.getString("mcastService.noStart"));
+        }
 
         byte[] data = XByteBuffer.createDataPackage((ChannelData)message);
         if (data.length>McastServiceImpl.MAX_PACKET_SIZE) {
@@ -509,6 +546,7 @@ public class McastService implements MembershipService,MembershipListener,Messag
         }
     }
 
+    @Override
     public int getSoTimeout() {
         return mcastSoTimeout;
     }
@@ -518,6 +556,7 @@ public class McastService implements MembershipService,MembershipListener,Messag
         properties.setProperty("mcastSoTimeout", String.valueOf(mcastSoTimeout));
     }
 
+    @Override
     public int getTtl() {
         return mcastTTL;
     }
@@ -526,6 +565,7 @@ public class McastService implements MembershipService,MembershipListener,Messag
         return payload;
     }
 
+    @Override
     public byte[] getDomain() {
         return domain;
     }
@@ -541,7 +581,9 @@ public class McastService implements MembershipService,MembershipListener,Messag
         if ( localMember != null ) {
             localMember.setPayload(payload);
             try {
-                if (impl != null) impl.send(false);
+                if (impl != null) {
+                    impl.send(false);
+                }
             }catch ( Exception x ) {
                 log.error(sm.getString("McastService.payload"), x);
             }
@@ -554,7 +596,9 @@ public class McastService implements MembershipService,MembershipListener,Messag
         if ( localMember != null ) {
             localMember.setDomain(domain);
             try {
-                if (impl != null) impl.send(false);
+                if (impl != null) {
+                    impl.send(false);
+                }
             }catch ( Exception x ) {
                 log.error(sm.getString("McastService.domain"), x);
             }
@@ -562,37 +606,52 @@ public class McastService implements MembershipService,MembershipListener,Messag
     }
 
     public void setDomain(String domain) {
-        if ( domain == null ) return;
-        if ( domain.startsWith("{") ) setDomain(Arrays.fromString(domain));
-        else setDomain(Arrays.convert(domain));
+        if ( domain == null ) {
+            return;
+        }
+        if ( domain.startsWith("{") ) {
+            setDomain(Arrays.fromString(domain));
+        } else {
+            setDomain(Arrays.convert(domain));
+        }
     }
 
+    @Override
     public Channel getChannel() {
         return channel;
     }
 
+    @Override
     public void setChannel(Channel channel) {
         this.channel = channel;
     }
 
     protected void setDefaults(Properties properties) {
         // default values
-        if (properties.getProperty("mcastPort") == null)
+        if (properties.getProperty("mcastPort") == null) {
             properties.setProperty("mcastPort","45564");
-        if (properties.getProperty("mcastAddress") == null)
+        }
+        if (properties.getProperty("mcastAddress") == null) {
             properties.setProperty("mcastAddress","228.0.0.4");
-        if (properties.getProperty("memberDropTime") == null)
+        }
+        if (properties.getProperty("memberDropTime") == null) {
             properties.setProperty("memberDropTime","3000");
-        if (properties.getProperty("mcastFrequency") == null)
+        }
+        if (properties.getProperty("mcastFrequency") == null) {
             properties.setProperty("mcastFrequency","500");
-        if (properties.getProperty("recoveryCounter") == null)
+        }
+        if (properties.getProperty("recoveryCounter") == null) {
             properties.setProperty("recoveryCounter", "10");
-        if (properties.getProperty("recoveryEnabled") == null)
+        }
+        if (properties.getProperty("recoveryEnabled") == null) {
             properties.setProperty("recoveryEnabled", "true");
-        if (properties.getProperty("recoverySleepTime") == null)
+        }
+        if (properties.getProperty("recoverySleepTime") == null) {
             properties.setProperty("recoverySleepTime", "5000");
-        if (properties.getProperty("localLoopbackDisabled") == null)
+        }
+        if (properties.getProperty("localLoopbackDisabled") == null) {
             properties.setProperty("localLoopbackDisabled", "false");
+        }
     }
 
     /**
