@@ -51,7 +51,6 @@ import org.apache.juli.logging.LogFactory;
  * The ConnectionPool uses a {@link PoolProperties} object for storing all the meta information about the connection pool.
  * As the underlying implementation, the connection pool uses {@link java.util.concurrent.BlockingQueue} to store active and idle connections.
  * A custom implementation of a fair {@link FairBlockingQueue} blocking queue is provided with the connection pool itself.
- * @version 1.0
  */
 public class ConnectionPool {
 
@@ -147,7 +146,7 @@ public class ConnectionPool {
      * Instantiate a connection pool. This will create connections if initialSize is larger than 0.
      * The {@link PoolProperties} should not be reused for another connection pool.
      * @param prop PoolProperties - all the properties for this connection pool
-     * @throws SQLException
+     * @throws SQLException Pool initialization error
      */
     public ConnectionPool(PoolConfiguration prop) throws SQLException {
         //setup quick access variables and pools
@@ -161,7 +160,7 @@ public class ConnectionPool {
      * If a connection is not retrieved, the Future must be cancelled in order for the connection to be returned
      * to the pool.
      * @return a Future containing a reference to the connection or the future connection
-     * @throws SQLException
+     * @throws SQLException Cannot use asynchronous connect
      */
     public Future<Connection> getConnectionAsync() throws SQLException {
         try {
@@ -206,7 +205,8 @@ public class ConnectionPool {
      * maxActive} connections a connection is returned immediately. If no
      * connection is available, the pool will attempt to fetch a connection for
      * {@link PoolProperties#maxWait maxWait} milliseconds.
-     *
+     * @param username The user name to use for the connection
+     * @param password The password for the connection
      * @return Connection - a java.sql.Connection/javax.sql.PooledConnection
      *         reflection proxy, wrapping the underlying object.
      * @throws SQLException
@@ -294,6 +294,9 @@ public class ConnectionPool {
         //fetch previously cached interceptor proxy - one per connection
         JdbcInterceptor handler = con.getHandler();
         if (handler==null) {
+            if (jmxPool != null) {
+                con.createMBean();
+            }
             //build the proxy handler
             handler = new ProxyConnection(this,con,getPoolProperties().isUseEquals());
             //set up the interceptor chain
@@ -353,8 +356,9 @@ public class ConnectionPool {
     /**
      * Creates and caches a {@link java.lang.reflect.Constructor} used to instantiate the proxy object.
      * We cache this, since the creation of a constructor is fairly slow.
+     * @param xa Use a XA connection
      * @return constructor used to instantiate the wrapper object
-     * @throws NoSuchMethodException
+     * @throws NoSuchMethodException Failed to get a constructor
      */
     public Constructor<?> getProxyConstructor(boolean xa) throws NoSuchMethodException {
         //cache the constructor
@@ -375,7 +379,9 @@ public class ConnectionPool {
      */
     protected void close(boolean force) {
         //are we already closed
-        if (this.closed) return;
+        if (this.closed) {
+            return;
+        }
         //prevent other threads from entering
         this.closed = true;
         //stop background thread
@@ -384,19 +390,20 @@ public class ConnectionPool {
         }
 
         /* release all idle connections */
-        BlockingQueue<PooledConnection> pool = (idle.size()>0)?idle:(force?busy:idle);
-        while (pool.size()>0) {
+        BlockingQueue<PooledConnection> pool = (!idle.isEmpty())?idle:(force?busy:idle);
+        while (!pool.isEmpty()) {
             try {
                 //retrieve the next connection
                 PooledConnection con = pool.poll(1000, TimeUnit.MILLISECONDS);
                 //close it and retrieve the next one, if one is available
                 while (con != null) {
                     //close the connection
-                    if (pool==idle)
+                    if (pool==idle) {
                         release(con);
-                    else
+                    } else {
                         abandon(con);
-                    if (pool.size()>0) {
+                    }
+                    if (!pool.isEmpty()) {
                         con = pool.poll(1000, TimeUnit.MILLISECONDS);
                     } else {
                         break;
@@ -407,9 +414,13 @@ public class ConnectionPool {
                     Thread.currentThread().interrupt();
                 }
             }
-            if (pool.size()==0 && force && pool!=busy) pool = busy;
+            if (pool.isEmpty() && force && pool!=busy) {
+                pool = busy;
+            }
+        }
+        if (this.getPoolProperties().isJmxEnabled()) {
+            this.jmxPool = null;
         }
-        if (this.getPoolProperties().isJmxEnabled()) this.jmxPool = null;
         PoolProperties.InterceptorDefinition[] proxies = getPoolProperties().getJdbcInterceptorsAsArray();
         for (int i=0; i<proxies.length; i++) {
             try {
@@ -450,7 +461,9 @@ public class ConnectionPool {
         initializePoolCleaner(properties);
 
         //create JMX MBean
-        if (this.getPoolProperties().isJmxEnabled()) createMBean();
+        if (this.getPoolProperties().isJmxEnabled()) {
+            createMBean();
+        }
 
         //Parse and create an initial set of interceptors. Letting them know the pool has started.
         //These interceptors will not get any connection.
@@ -465,7 +478,9 @@ public class ConnectionPool {
                 interceptor.poolStarted(this);
             }catch (Exception x) {
                 log.error("Unable to inform interceptor of pool start.",x);
-                if (jmxPool!=null) jmxPool.notify(org.apache.tomcat.jdbc.pool.jmx.ConnectionPool.NOTIFY_INIT, getStackTrace(x));
+                if (jmxPool!=null) {
+                    jmxPool.notify(org.apache.tomcat.jdbc.pool.jmx.ConnectionPool.NOTIFY_INIT, getStackTrace(x));
+                }
                 close(true);
                 SQLException ex = new SQLException();
                 ex.initCause(x);
@@ -483,7 +498,9 @@ public class ConnectionPool {
         } catch (SQLException x) {
             log.error("Unable to create initial connections of pool.", x);
             if (!poolProperties.isIgnoreExceptionOnPreLoad()) {
-                if (jmxPool!=null) jmxPool.notify(org.apache.tomcat.jdbc.pool.jmx.ConnectionPool.NOTIFY_INIT, getStackTrace(x));
+                if (jmxPool!=null) {
+                    jmxPool.notify(org.apache.tomcat.jdbc.pool.jmx.ConnectionPool.NOTIFY_INIT, getStackTrace(x));
+                }
                 close(true);
                 throw x;
             }
@@ -538,6 +555,7 @@ public class ConnectionPool {
         }
     }
 
+
 //===============================================================================
 //         CONNECTION POOLING IMPL LOGIC
 //===============================================================================
@@ -549,8 +567,9 @@ public class ConnectionPool {
      * @param con PooledConnection
      */
     protected void abandon(PooledConnection con) {
-        if (con == null)
+        if (con == null) {
             return;
+        }
         try {
             con.lock();
             String trace = con.getStackTrace();
@@ -577,10 +596,12 @@ public class ConnectionPool {
      * @param con PooledConnection
      */
     protected void suspect(PooledConnection con) {
-        if (con == null)
+        if (con == null) {
             return;
-        if (con.isSuspect())
+        }
+        if (con.isSuspect()) {
             return;
+        }
         try {
             con.lock();
             String trace = con.getStackTrace();
@@ -601,8 +622,9 @@ public class ConnectionPool {
      * @param con PooledConnection
      */
     protected void release(PooledConnection con) {
-        if (con == null)
+        if (con == null) {
             return;
+        }
         try {
             con.lock();
             if (con.release()) {
@@ -618,7 +640,10 @@ public class ConnectionPool {
         // we could have threads stuck in idle.poll(timeout) that will never be
         // notified
         if (waitcount.get() > 0) {
-            idle.offer(create(true));
+            if (!idle.offer(create(true))) {
+                log.warn("Failed to add a new connection to the pool after releasing a connection " +
+                        "when at least one thread was waiting for a connection.");
+            }
         }
     }
 
@@ -626,8 +651,10 @@ public class ConnectionPool {
      * Thread safe way to retrieve a connection from the pool
      * @param wait - time to wait, overrides the maxWait from the properties,
      * set to -1 if you wish to use maxWait, 0 if you wish no wait time.
-     * @return PooledConnection
-     * @throws SQLException
+     * @param username The user name to use for the connection
+     * @param password The password for the connection
+     * @return a connection
+     * @throws SQLException Failed to get a connection
      */
     private PooledConnection borrowConnection(int wait, String username, String password) throws SQLException {
 
@@ -645,7 +672,9 @@ public class ConnectionPool {
                 //configure the connection and return it
                 PooledConnection result = borrowConnection(now, con, username, password);
                 borrowedCount.incrementAndGet();
-                if (result!=null) return result;
+                if (result!=null) {
+                    return result;
+                }
             }
 
             //if we get here, see if we need to create one
@@ -713,14 +742,20 @@ public class ConnectionPool {
      * Creates a JDBC connection and tries to connect to the database.
      * @param now timestamp of when this was called
      * @param notUsed Argument not used
+     * @param username The user name to use for the connection
+     * @param password The password for the connection
      * @return a PooledConnection that has been connected
-     * @throws SQLException
+     * @throws SQLException Failed to get a connection
      */
     protected PooledConnection createConnection(long now, PooledConnection notUsed, String username, String password) throws SQLException {
         //no connections where available we'll create one
         PooledConnection con = create(false);
-        if (username!=null) con.getAttributes().put(PooledConnection.PROP_USER, username);
-        if (password!=null) con.getAttributes().put(PooledConnection.PROP_PASSWORD, password);
+        if (username!=null) {
+            con.getAttributes().put(PooledConnection.PROP_USER, username);
+        }
+        if (password!=null) {
+            con.getAttributes().put(PooledConnection.PROP_PASSWORD, password);
+        }
         boolean error = false;
         try {
             //connect and validate the connection
@@ -744,8 +779,9 @@ public class ConnectionPool {
             } //end if
         } catch (Exception e) {
             error = true;
-            if (log.isDebugEnabled())
+            if (log.isDebugEnabled()) {
                 log.debug("Unable to create a new JDBC connection.", e);
+            }
             if (e instanceof SQLException) {
                 throw (SQLException)e;
             } else {
@@ -766,7 +802,9 @@ public class ConnectionPool {
      * Validates and configures a previously idle connection
      * @param now - timestamp
      * @param con - the connection to validate and configure
-     * @return con
+     * @param username The user name to use for the connection
+     * @param password The password for the connection
+     * @return a connection
      * @throws SQLException if a validation error happens
      */
     protected PooledConnection borrowConnection(long now, PooledConnection con, String username, String password) throws SQLException {
@@ -850,7 +888,7 @@ public class ConnectionPool {
     }
     /**
      * Terminate the current transaction for the given connection.
-     * @param con
+     * @param con The connection
      * @return <code>true</code> if the connection TX termination succeeded
      *         otherwise <code>false</code>
      */
@@ -859,10 +897,14 @@ public class ConnectionPool {
             if (Boolean.FALSE.equals(con.getPoolProperties().getDefaultAutoCommit())) {
                 if (this.getPoolProperties().getRollbackOnReturn()) {
                     boolean autocommit = con.getConnection().getAutoCommit();
-                    if (!autocommit) con.getConnection().rollback();
+                    if (!autocommit) {
+                        con.getConnection().rollback();
+                    }
                 } else if (this.getPoolProperties().getCommitOnReturn()) {
                     boolean autocommit = con.getConnection().getAutoCommit();
-                    if (!autocommit) con.getConnection().commit();
+                    if (!autocommit) {
+                        con.getConnection().commit();
+                    }
                 }
             }
             return true;
@@ -877,16 +919,29 @@ public class ConnectionPool {
      * Determines if a connection should be closed upon return to the pool.
      * @param con - the connection
      * @param action - the validation action that should be performed
-     * @return true if the connection should be closed
+     * @return <code>true</code> if the connection should be closed
      */
     protected boolean shouldClose(PooledConnection con, int action) {
-        if (con.getConnectionVersion() < getPoolVersion()) return true;
-        if (con.isDiscarded()) return true;
-        if (isClosed()) return true;
-        if (!con.validate(action)) return true;
-        if (!terminateTransaction(con)) return true;
-        if (con.isMaxAgeExpired()) return true;
-        else return false;
+        if (con.getConnectionVersion() < getPoolVersion()) {
+            return true;
+        }
+        if (con.isDiscarded()) {
+            return true;
+        }
+        if (isClosed()) {
+            return true;
+        }
+        if (!con.validate(action)) {
+            return true;
+        }
+        if (!terminateTransaction(con)) {
+            return true;
+        }
+        if (con.isMaxAgeExpired()) {
+            return true;
+        } else {
+            return false;
+        }
     }
 
     /**
@@ -951,11 +1006,15 @@ public class ConnectionPool {
     /**
      * Determines if a connection should be abandoned based on
      * {@link PoolProperties#abandonWhenPercentageFull} setting.
-     * @return true if the connection should be abandoned
+     * @return <code>true</code> if the connection should be abandoned
      */
     protected boolean shouldAbandon() {
-        if (!poolProperties.isRemoveAbandoned()) return false;
-        if (poolProperties.getAbandonWhenPercentageFull()==0) return true;
+        if (!poolProperties.isRemoveAbandoned()) {
+            return false;
+        }
+        if (poolProperties.getAbandonWhenPercentageFull()==0) {
+            return true;
+        }
         float used = busy.size();
         float max  = poolProperties.getMaxActive();
         float perc = poolProperties.getAbandonWhenPercentageFull();
@@ -967,7 +1026,9 @@ public class ConnectionPool {
      */
     public void checkAbandoned() {
         try {
-            if (busy.size()==0) return;
+            if (busy.isEmpty()) {
+                return;
+            }
             Iterator<PooledConnection> locked = busy.iterator();
             int sto = getPoolProperties().getSuspectTimeout();
             while (locked.hasNext()) {
@@ -977,8 +1038,9 @@ public class ConnectionPool {
                     con.lock();
                     //the con has been returned to the pool or released
                     //ignore it
-                    if (idle.contains(con) || con.isReleased())
+                    if (idle.contains(con) || con.isReleased()) {
                         continue;
+                    }
                     long time = con.getTimestamp();
                     long now = System.currentTimeMillis();
                     if (shouldAbandon() && (now - time) > con.getAbandonTimeout()) {
@@ -992,8 +1054,9 @@ public class ConnectionPool {
                     } //end if
                 } finally {
                     con.unlock();
-                    if (setToNull)
+                    if (setToNull) {
                         con = null;
+                    }
                 }
             } //while
         } catch (ConcurrentModificationException e) {
@@ -1014,7 +1077,9 @@ public class ConnectionPool {
     public void checkIdle(boolean ignoreMinSize) {
 
         try {
-            if (idle.size()==0) return;
+            if (idle.isEmpty()) {
+                return;
+            }
             long now = System.currentTimeMillis();
             Iterator<PooledConnection> unlocked = idle.iterator();
             while ( (ignoreMinSize || (idle.size()>=getPoolProperties().getMinIdle())) && unlocked.hasNext()) {
@@ -1023,8 +1088,9 @@ public class ConnectionPool {
                 try {
                     con.lock();
                     //the con been taken out, we can't clean it up
-                    if (busy.contains(con))
+                    if (busy.contains(con)) {
                         continue;
+                    }
                     long time = con.getTimestamp();
                     if (shouldReleaseIdle(now, con, time)) {
                         releasedIdleCount.incrementAndGet();
@@ -1036,8 +1102,9 @@ public class ConnectionPool {
                     } //end if
                 } finally {
                     con.unlock();
-                    if (setToNull)
+                    if (setToNull) {
                         con = null;
+                    }
                 }
             } //while
         } catch (ConcurrentModificationException e) {
@@ -1050,8 +1117,11 @@ public class ConnectionPool {
 
 
     protected boolean shouldReleaseIdle(long now, PooledConnection con, long time) {
-        if (con.getConnectionVersion() < getPoolVersion()) return true;
-        else return (con.getReleaseTime()>0) && ((now - time) > con.getReleaseTime()) && (getSize()>getPoolProperties().getMinIdle());
+        if (con.getConnectionVersion() < getPoolVersion()) {
+            return true;
+        } else {
+            return (con.getReleaseTime()>0) && ((now - time) > con.getReleaseTime()) && (getSize()>getPoolProperties().getMinIdle());
+        }
     }
 
     /**
@@ -1059,16 +1129,20 @@ public class ConnectionPool {
      */
     public void testAllIdle() {
         try {
-            if (idle.size()==0) return;
+            if (idle.isEmpty()) {
+                return;
+            }
             Iterator<PooledConnection> unlocked = idle.iterator();
             while (unlocked.hasNext()) {
                 PooledConnection con = unlocked.next();
                 try {
                     con.lock();
                     //the con been taken out, we can't clean it up
-                    if (busy.contains(con))
+                    if (busy.contains(con)) {
                         continue;
+                    }
                     if (!con.validate(PooledConnection.VALIDATE_IDLE)) {
+                        releasedIdleCount.incrementAndGet();
                         idle.remove(con);
                         release(con);
                     }
@@ -1115,10 +1189,13 @@ public class ConnectionPool {
 
     /**
      * Create a new pooled connection object. Not connected nor validated.
+     * @param incrementCounter <code>true</code> to increment the connection count
      * @return a pooled connection object
      */
     protected PooledConnection create(boolean incrementCounter) {
-        if (incrementCounter) size.incrementAndGet();
+        if (incrementCounter) {
+            size.incrementAndGet();
+        }
         PooledConnection con = new PooledConnection(getPoolProperties(), this);
         return con;
     }
@@ -1146,7 +1223,7 @@ public class ConnectionPool {
 
     /**
      * Hook to perform final actions on a pooled connection object once it has been disconnected and will be discarded
-     * @param con
+     * @param con The connection
      */
     protected void finalize(PooledConnection con) {
         JdbcInterceptor handler = con.getHandler();
@@ -1158,7 +1235,8 @@ public class ConnectionPool {
 
     /**
      * Hook to perform final actions on a pooled connection object once it has been disconnected and will be discarded
-     * @param con
+     * @param con The connection
+     * @param finalizing <code>true</code> if finalizing the connection
      */
     protected void disconnectEvent(PooledConnection con, boolean finalizing) {
         JdbcInterceptor handler = con.getHandler();
@@ -1257,7 +1335,7 @@ public class ConnectionPool {
     }
 
     /**
-     * Tread safe wrapper around a future for the regular queue
+     * Thread safe wrapper around a future for the regular queue
      * This one retrieves the pooled connection object
      * and performs the initialization according to
      * interceptors and validation rules.
@@ -1314,7 +1392,9 @@ public class ConnectionPool {
         public Connection get(long timeout, TimeUnit unit) throws InterruptedException, ExecutionException, TimeoutException {
             PooledConnection pc = this.pc!=null?this.pc:pcFuture.get(timeout,unit);
             if (pc!=null) {
-                if (result!=null) return result;
+                if (result!=null) {
+                    return result;
+                }
                 if (configured.compareAndSet(false, true)) {
                     try {
                         pc = borrowConnection(System.currentTimeMillis(),pc, null, null);
@@ -1328,7 +1408,9 @@ public class ConnectionPool {
                     //if we reach here, another thread is configuring the actual connection
                     latch.await(timeout,unit); //this shouldn't block for long
                 }
-                if (result==null) throw new ExecutionException(cause);
+                if (result==null) {
+                    throw new ExecutionException(cause);
+                }
                 return result;
             } else {
                 return null;
@@ -1398,7 +1480,7 @@ public class ConnectionPool {
             cleaner.cancel();
             if (poolCleanTimer != null) {
                 poolCleanTimer.purge();
-                if (cleaners.size() == 0) {
+                if (cleaners.isEmpty()) {
                     poolCleanTimer.cancel();
                     poolCleanTimer = null;
                 }
@@ -1449,13 +1531,16 @@ public class ConnectionPool {
             } else if (!pool.isClosed()) {
                 try {
                     if (pool.getPoolProperties().isRemoveAbandoned()
-                            || pool.getPoolProperties().getSuspectTimeout() > 0)
+                            || pool.getPoolProperties().getSuspectTimeout() > 0) {
                         pool.checkAbandoned();
+                    }
                     if (pool.getPoolProperties().getMinIdle() < pool.idle
-                            .size())
+                            .size()) {
                         pool.checkIdle();
-                    if (pool.getPoolProperties().isTestWhileIdle())
+                    }
+                    if (pool.getPoolProperties().isTestWhileIdle()) {
                         pool.testAllIdle();
+                    }
                 } catch (Exception x) {
                     log.error("", x);
                 }
